      program SIMair_LOOP
c------------------------------------------------------------------------------- 
c RAINS-ASIA Lagrangian model - Version 4.0
c
c G. Calori, Feb '99
c On the basis of ATMOS-1 (Arndt & Carmichael, 95) and UR-BAT (Calori, 96) codes
c
c S. Guttikinda, Dec 2009
c Generates grid 2 grid matrix for SIM-air for
c Four Pollutants = PM coarse; PM fine; sulfates; and nitrates
c Two sectors = area, point
c-------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc' 
      
      call SETUP
      call READ_METEO
       
      do isector = 1,2 		! area, point
         ihte = isector - 1	! 0 for area, 1 for point

      do ipoll = 1,4		! pm coarse, pm fine, so2, nox

        call POLL_PARAMS
        if(ihte.eq.0) write(io_msg,*) 'Computing for Area Emissions'
        if(ihte.eq.1) write(io_msg,*) 'Computing for Point Emissions' 
        write(io_msg,'(/,a,/)') '--- Computational phase'

      do imatrix = 1,ngrids
         write(io_msg,*) '--- Running for grid number = ', imatrix
         write(io_msg,*) source_lon(imatrix), source_lat(imatrix)
 
        call ATMOS_MAIN

      enddo
      enddo
      enddo

      stop
      end






      subroutine ATMOS_MAIN
c------------------------------------------------------------------------------- 
c RAINS-ASIA Lagrangian model - Version 2.1
c
c G. Calori, Feb '99
c On the basis of ATMOS-1 (Arndt & Carmichael, 95) and UR-BAT (Calori, 96) codes
c
c S. Guttikinda, Nov 2009
c Removed netcdf and ioapi links and uses meteorology for a point only
c-------------------------------------------------------------------------------
      implicit none
      
      include 'atmos40.inc' 

      integer julian
      integer i,j, itraj, ibr
 
c ---	setup phase

      call RESET_TRAJS 		! initialize traj parameters 
 
      call RESET_CONC 		! initialize conc. fields

      hour = dttransp
      day = 1
      month = 1
      call START_TRAJS
   
c ---	main time loop

      do month = 1, 12
      do day = 1, ndaym(month)
      do hour = dttransp,24,dttransp
 
            julday = JULIAN (month, day)
 
c	set day/night flags and day/night transition flags 
 
            call SET_DN_FLAGS
 
c	trajs & brnch stats
 
            ntrajtot = 0 
            nbrtot = 0 
            do itraj = 1,maxtraj 
              if (nbranches(itraj) .gt. 0) then 
                ntrajtot = ntrajtot + 1
                do ibr = 1,nbranches(itraj) 
                  if (br_layer(ibr,itraj) .ne. 0) nbrtot = nbrtot+1 
                enddo 
              endif 
            enddo 

c            if (.true.)
c     +        write(io_msg,'(a,2(i2.2,a),i4,a,i2.2,a,i6,a,i8,a)')
c     +          '- UTC clock: ', 
c     +          day, '-', month, '-', year, ' ', hour,
c     +          ', ', ntrajtot, ' trajs ', nbrtot, ' branches'

c	advects each branch of each traj 
                                 
            call ADVECT 
 
c	b.l. height calc for each branch of each traj
 
            call SET_HMIX
                      
c	puffs branching for all trajs
 
            call BRANCH

c	updates masses of each branch of each traj;  
c	updates conc and dep maps 
 
            call CONCAL
 
c	terminate traj older than fixed time
 
            call TERM_TRAJ 
 
c	start a new set of trajs (time res. specified in the emission file)
 
            call START_TRAJS
 
      enddo 	! hour loop
      enddo		! day loop
      enddo 	! month loop

      call CONC_AVERAGE 
      print *, iwrite
         write(io_out,99) 
     +         ((conc_avg(i + 5, j + 5, iwrite),i=1,nx),j=ny,1,-1)
  99     format(625(f8.2,2x))

c --- termination phase

      return
      end                                                                        
 
 


      subroutine SETUP
c------------------------------------------------------------------------------
c G. Calori, 24.2.99
c
c PURPOSE: Setup phase of atmos-4.0
c 
c S. Guttikunda, December 2009
c Updated for common input and emissions setup
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'
      integer i, j, isou, ii, jj

c	i/o units

      io_ini = 1
      io_out = 2
      io_msg = 6

      write(io_msg,'(//,a,/)')
     +  '=== ' // progname // ' ==='
      write(io_msg,'(3x,a)')
     +  'RAINS-Asia Lagrangian multi-layer model',
     +  'Modified for SIM-air tool',
     +  'Output will be a transfer matrix'
      
      write(io_msg,'(/,a,/)') '--- Setup phase'

c	get some calculation parameters

      open (unit=io_ini, file='inifile', status='old') 

      call SKIP_COMMENT_LINES (io_ini, '!')
      read(io_ini,*) year
      read(io_ini,*) trajdur	! trajectories duration (hr)
      read(io_ini,*) dttransp	! transport time step (hr)
      read(io_ini,*) monthb	! average period starting - 1 for yearly
      read(io_ini,*) monthe	! average period ending - 12 for yearly
    
      ndaym(1) = 31
      If (mod(year,4).eq.0) Then
        ndaym(2) = 29
          Else
        ndaym(2) = 28
      End If
      ndaym(3) = 31
      ndaym(4) = 30
      ndaym(5) = 31
      ndaym(6) = 30
      ndaym(7) = 31
      ndaym(8) = 31
      ndaym(9) = 30
      ndaym(10) = 31
      ndaym(11) = 30
      ndaym(12) = 31 

      call SKIP_COMMENT_LINES (io_ini, '!')
      read(io_ini,*) xgrid	! south west corner - longitude (degrees) 
      read(io_ini,*) ygrid	! south west corner - latitude (degrees) 
      read(io_ini,*) dx_c	! grid resolution - longitudes (degrees) 
      read(io_ini,*) dy_c	! grid resolution - latitudes (degrees) 
      read(io_ini,*) nx		! number of grids - longitudes 
      read(io_ini,*) ny		! number of grids - latitudes 
      
      ngrids = nx * ny

      xmin_o = xgrid + dx_c/2	! minimum for output grid (center)
      ymin_o = ygrid + dy_c/2	! minimum for output grid (center)
      xmax_o = xmin_o + (nx-1) * dx_c
      ymax_o = ymin_o + (ny-1) * dy_c
 
      nx_c = nx + 10		! buffer of 5 grids on all sides
      ny_c = ny + 10		! buffer of 5 grids on all sides
      xmin_c = xmin_o - dx_c*5	! minimum for model grid (center)
      ymin_c = ymin_o - dy_c*5 	! minimum for model grid (center)
      xmax_c = xmin_c + dx_c * (nx_c-1)
      ymax_c = ymin_c + dy_c * (ny_c-1)

      do isou = 1,ngrids 
        source_lon(isou) = 0.
        source_lat(isou) = 0.
        emiss(isou) = 0.
      enddo 
      
      do jj = 1,ny
      do ii = 1,nx
         source_lon(ii+(jj-1)*nx) = xmin_o + dx_c * (ii - 1)
         source_lat(ii+(jj-1)*nx) = ymax_o - dy_c * (jj - 1)
         emiss(ii+(jj-1)*nx) = 3000.* 1.e+06 /
     +                        (8760. / dttransp)
      enddo
      enddo

      sh0_diff = (dx_c+dy_c)/2*80.0/10	! initial horiz. sigma for diffuse sources (m) 
      sh0_lps  = (dx_c+dy_c)/2*80.0/5	! initial horiz. sigma for LPS (m)
      
      min_mass = 0.1		! puff minimum mass (% of initial release) 

      call SKIP_COMMENT_LINES (io_ini, '!')
      read(io_ini,*) dry_vd_p10	! dry deposition coefficient for corase PM 
      read(io_ini,*) dry_vd_p25	! dry deposition coefficient for fine PM 
      read(io_ini,*) dry_vd_so2	! dry deposition coefficient for SO2 
      read(io_ini,*) dry_vd_so4	! dry deposition coefficient for SO4 
      read(io_ini,*) dry_vd_nox	! dry deposition coefficient for NOx 
      read(io_ini,*) dry_vd_no3	! dry deposition coefficient for NO3

      call SKIP_COMMENT_LINES (io_ini, '!')
      read(io_ini,*) wet_vd_p10	! wet deposition coefficient for corase PM 
      read(io_ini,*) wet_vd_p25	! wet deposition coefficient for fine PM 
      read(io_ini,*) wet_vd_so2	! wet deposition coefficient for SO2 
      read(io_ini,*) wet_vd_so4	! wet deposition coefficient for SO4 
      read(io_ini,*) wet_vd_nox	! wet deposition coefficient for NOx 
      read(io_ini,*) wet_vd_no3	! wet deposition coefficient for NO3 

      call SKIP_COMMENT_LINES (io_ini, '!')
      read(io_ini,*) kr_so2	! Reaction rate for SO2 
      read(io_ini,*) kr_nox	! Reaction rate for NOx 

      close(io_ini) 

c	set some calculation parameters
c     computes coslat (cosine of lat) at gridpoints

      do i = 1,ny_c
        coslat(i) = cos(d2r*(ymin_c + real(i-1) * dy_c))
      enddo

c     values of exp(-x**2/2), for x = 0, 0.1, 0.2, 0.3, .. 4

      do i = 1,41
        ex2(i) = exp(-(0.1 * real(i-1))**2 / 2.)
      enddo

c	multip. factor for sigma-h

      lr_sh_param = 0.5 * dttransp * 3600.

c	finished setting up parameters

      write(io_msg,'(/,a,i3)') 'Output conc./dep. archive:'
      write(io_msg,'(a,2f9.3,i5)') 
     +  'x - min coord., grid step size, # of points: ', 
     +  xmin_o, dx_c, nx
      write(io_msg,'(a,2f9.3,i5)') 
     +  'y - min coord., grid step size, # of points: ', 
     +  ymin_o, dy_c, ny


      return
      end
      




 


      subroutine POLL_PARAMS
c------------------------------------------------------------------------------
c S. Guttikunda, November 2009
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'
      character outname*16

c	species names

      if(ipoll.eq.1) then
        write(io_msg,'(//,a,/)')
     +     'Pollutant Selected is Particulates 10.0'
        dry_vd_1 = dry_vd_p10
        dry_vd_2 = 0.
        wet_vd_1 = wet_vd_p10
        wet_vd_2 = 0.
        kr = 0.
        iwrite = 1
        if(ihte.eq.0) write(outname,'(a16)') 'pmc.larea.matrix'
        if(ihte.eq.1) write(outname,'(a16)') 'pmc.point.matrix'
        open (unit=io_out, file=outname, status='unknown') 
      endif

      if(ipoll.eq.2) then
        write(io_msg,'(//,a,/)')
     +     'Pollutant Selected is Particulates 2.5'
        dry_vd_1 = dry_vd_p25
        dry_vd_2 = 0.
        wet_vd_1 = wet_vd_p25
        wet_vd_2 = 0.
        kr = 0.
        iwrite = 1
        if(ihte.eq.0) write(outname,'(a16)') 'pmf.larea.matrix'
        if(ihte.eq.1) write(outname,'(a16)') 'pmf.point.matrix'
        open (unit=io_out, file=outname, status='unknown')
      endif

      if(ipoll.eq.3) then
        write(io_msg,'(//,a,/)')
     +     'Pollutant Selected is Sulfur Dioxide'
        dry_vd_1 = dry_vd_so2
        dry_vd_2 = dry_vd_so4
        wet_vd_1 = wet_vd_so2
        wet_vd_2 = wet_vd_so4
        kr = kr_so2
        iwrite = 2      
        if(ihte.eq.0) write(outname,'(a16)') 'so4.larea.matrix'
        if(ihte.eq.1) write(outname,'(a16)') 'so4.point.matrix'
        open (unit=io_out, file=outname, status='unknown')
      endif

      if(ipoll.eq.4) then
        write(io_msg,'(//,a,/)')
     +     'Pollutant Selected is Nitrogen Oxides'
        dry_vd_1 = dry_vd_nox
        dry_vd_2 = dry_vd_no3
        wet_vd_1 = wet_vd_nox
        wet_vd_2 = wet_vd_no3
        kr = kr_nox
        iwrite = 2
        if(ihte.eq.0) write(outname,'(a16)') 'no3.larea.matrix'
        if(ihte.eq.1) write(outname,'(a16)') 'no3.point.matrix'
        open (unit=io_out, file=outname, status='unknown')
      endif

c	partition of emissions among species

      if(ipoll.eq.3.or.ipoll.eq.4) then
        spec_emiperc(1) = 95.                   ! % of emission emitted as SO2 or NOx
        spec_emiperc(2) = 100. - spec_emiperc(1)        ! % of emission emitted as SO4 or NO3
      else
        spec_emiperc(1) = 100.                   ! % of emission - no secondary
        spec_emiperc(2) = 100. - spec_emiperc(1)
      endif
      
c	mass unit conversion factors

      if(ipoll.eq.3) then
         massfact(1) = 1./2.    ! from SO2 to S
         massfact(2) = 1./3.    ! from SO4 to S
      else
         massfact(1) = 1.
         massfact(2) = 1.
      endif

      return
      end














 
      subroutine ADVECT 
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c Adapted from ATMOS 1.0 model (Arndt & Carmichael, 95), 
c based on BAT model (Heffter, 83)
c
c PURPOSE: Advects each branch of each traj 
c          with modified Euler advection algorithm 
c
c LAST REV.:
c S. Guttikinda, Nov 2009
c Removed netcdf and ioapi links and uses meteorology for a point only
c------------------------------------------------------------------------------
      implicit none
      
      include 'atmos40.inc' 

      integer neuler
      parameter (neuler = 2)	! # of Euler iterations 
      real msec2deghr
      parameter (msec2deghr = 3600. / deg2m) 	! factor from [m/sec] to [deg lat/hr] 
  
      integer itraj, ibranch, ilayer, ieuler, jlat1, jlat2, inew, jnew
      real lon_new, lon_old, lat_new, lat_old, fact, latfact
      real xwind, ywind
      logical stopit
      
      fact = msec2deghr * dttransp
 
      do itraj = 1,maxtraj		! for each traj  
 
        do ibranch = 1,nbranches(itraj)   ! for each branch 
 
          if (br_layer(ibranch,itraj) .gt. 0) then      ! if current branch is active 
 
c 	set the layer index to be used for advection 
                                                                                
            ilayer = br_layer(ibranch,itraj) 
            if (ilayer .eq. 6) ilayer = 1	! surf 
            if (ilayer .eq. 4) ilayer = 2	! boundary 
            if (ilayer .eq. 5) ilayer = 3	! upper 
 
c	store current coordinates 
 
            br_lat_old(ibranch,itraj) = br_lat(ibranch,itraj) 
            br_lon_old(ibranch,itraj) = br_lon(ibranch,itraj) 
 
            lon_old = br_lon(ibranch,itraj) 
            lat_old = br_lat(ibranch,itraj) 
            lon_new = lon_old 
            lat_new = lat_old 
 
            ieuler = 1 		! current Euler iteration
            stopit = .false.	! branch is inside calc. domain
            
            do while (ieuler .le. neuler .and. .not. stopit) 
 
c	get wind at current point

              if(uwind(month,day,hour).ne.0.) then
                xwind = uwind(month, day, hour)
                ywind = vwind(month, day, hour)
              endif
c              print *, 'xwind ywind', xwind, ywind

c	compute latitude factor
     
              jlat1 = int((lat_new - ymin_c) / dy_c) + 1
              if (jlat1 .lt. ny_c) then
                jlat2 = jlat1 + 1
              else
                jlat2 = jlat1
              endif
              latfact = 0.5 * (coslat(jlat1) + coslat(jlat2))
               
c 	updates branch coordinates 
  
              lon_new = lon_old + xwind * fact / latfact
              lat_new = lat_old + ywind * fact 

c	boundaries check

              inew = int((lon_new - xmin_c) / dx_c) + 1
              jnew = int((lat_new - ymin_c) / dy_c) + 1
 
              if (inew .lt. 1 .or. inew .gt. nx_c .or. 
     +            jnew .lt. 1 .or. jnew .gt. ny_c) then 
 
c	branch terminated due to out of boundary 
 
                br_layer(ibranch,itraj) = 0 
                stopit = .true.			! stop Euler iterations 
 
              else 
  
                lon_new = (lon_old + lon_new) * .5 
                lat_new = (lat_old + lat_new) * .5 
                ieuler = ieuler + 1

              endif 
 
            enddo 
 
c 	store new coord and update traj life time (hr)
 
            br_lon(ibranch,itraj) = lon_new 
            br_lat(ibranch,itraj) = lat_new 
 
          endif 

        enddo		! next branch 
 
        life_traj(itraj) = life_traj(itraj) + dttransp

      enddo		! next traj 
 
      return 
      end                                                                        









      subroutine CONCAL
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c Adapted from ATMOS 1.0 model (Arndt & Carmichael, 95), 
c based on BAT model (Heffter, 83)
c
c PURPOSE: Updates masses of each branch of each traj, according to
c          chemistry & depositions; updates conc and dep maps.
c
c LAST REV.:
c S. Guttikunda, November 2009
c Single output file with averages per month
c Deposition and reaction rates are fixed, read in setup subroutine
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'

      real gfact			! factor in Gaussian distrib
      parameter (gfact = pi2 * 1.e-06)	! grams -> ugrams
      real so2_ptresh
      parameter (so2_ptresh = 1.e-5)	! (changed from 1e-6)
      
      integer itraj, ibranch, ilayer, ib
      integer iland, iurbrur, ipuff, jpuff, iradius, jradius, 
     +        i, j, i1, i2, j1, j2, ii1, ii2, jj1, jj2, ie, istab, klay
      real inv_dx_c, inv_dy_c, inv_dx_c_m, inv_dy_c_m
      real dt, t, emi100th, blon, blat, z, p, x, y, r,
     +     dx, dy, deltax, uu, vv, u, sh, prad,
     +     c_vol, concso2, concso4
      real dso2, dso4, 
     +     so2_old, so4_old, so2_new, so4_new,
     +     expso2, expso4, dd, sdn, sdo, eie	!, sdoexp, edt, temi
      real tmp1, tmp2, vdso2z, vdso4z, dryso2, dryso4, wetso2, wetso4
      real distrib(maxx_c,maxy_c), ndistrib(maxx_c,maxy_c), 
     +     sum, sumtrash, sumtot, invsum, fact
      real d2, d4, w2, w4, smass     
      logical reachsrf


      navgstep(month) = navgstep(month) + 1	! update # of averaging steps

c	set calculations constants

      dt = dttransp * 3600.	! mass eq. integration time step (sec)

      inv_dx_c = 1. / dx_c
      inv_dy_c = 1. / dy_c
      inv_dx_c_m = inv_dx_c / deg2m		! inverse of conc grid res. in m (x)
      inv_dy_c_m = inv_dy_c / deg2m		! inverse of conc grid res. in m (y)

      do itraj = 1,maxtraj            ! for each traj

        emi100th = traj_emiss(itraj) * 0.01	! emission carried by traj (grams/100.)
        t = life_traj(itraj) * 3600		! fly time in sec (since traj started)

        do ibranch = 1,nbranches(itraj)               ! for each branch
     
          ilayer = br_layer(ibranch,itraj)            ! current layer index

          if (ilayer .ne. 0) then      ! if current branch is active

            blon = br_lon(ibranch,itraj)
            blat = br_lat(ibranch,itraj)
            z = 0.

c 	computes conc. grid indexes of puff centre

            ipuff = nint((blon - xmin_c) * inv_dx_c) + 1
            jpuff = nint((blat - ymin_c) * inv_dy_c) + 1

            if (ipuff .ge. 1 .and. ipuff .le. nx_c .and.
     +          jpuff .ge. 1 .and. jpuff .le. ny_c) then	! puff inside target grid
                     
              iland = 1			! always land

              iurbrur = 1		! always urban
 
c 	see if current layer is in contact with the terrain 
c	and compute its mixing height

              goto (1,2,3,4,5,6) ilayer

    1         continue            ! surf (night)
                reachsrf = .true.
                z = sltop
                if (iurbrur .eq. 1) z = nultop
                goto 9
    2         continue            ! bndry (day)
                reachsrf = .true.
                z = br_blt(ibranch,itraj)
                goto 9
    3         continue            ! upper (day)
                reachsrf = .false.
                z = ultop - br_blt(ibranch,itraj)
                goto 9
    4         continue            ! boundary (night)
                reachsrf = .false.
                if (iurbrur .eq. 1) then
                  z = br_blt(ibranch,itraj) - nultop
                else
                  z = br_blt(ibranch,itraj) - sltop
                endif
                goto 9
    5         continue            ! upper (night)
                reachsrf = .false.
                  z = ultop - br_blt(ibranch,itraj)
                goto 9
    6           continue            ! surf (day)
                reachsrf = .true.
                z = sltop

    9         if (z .lt. sltop) z = sltop

c	get precip. rate (mm/hr)

              p = precip(month,day,hour)

c	compute exp. destruction terms for mass balance eq. (s-1)

c	... reaction rate

              dso2 = kr
              dso4 = 0.

c 	... dry dep vels (m/s) (layers in contact with the surface)

              vdso2z = 0.
              vdso4z = 0.
              if (reachsrf) then
                vdso2z = dry_vd_1 / z
                vdso4z = dry_vd_2 / z
                dso2 = dso2 + vdso2z
                dso4 = dso4 + vdso4z
              endif

c	... scavenging rates

              if (p .gt. 0.) then
                dso2 = dso2 + wet_vd_1 * p
                dso4 = dso4 + wet_vd_2 * p**0.65
              endif

c 	updates puff SO2 and SO4 mass %

              so2_old = br_pmass(ibranch,itraj,1)	! store current % of SO2 mass
              so4_old = br_pmass(ibranch,itraj,2)	! store current % of SO4 mass

              expso2 = exp(-dso2 * dt)
              so2_new = so2_old * expso2 
              br_pmass(ibranch,itraj,1) = so2_new	! updates % of SO2 mass

              expso4 = exp(-dso4 * dt)
              dd = dso4 - dso2
              if (dd .ne. 0.) then
                sdn = (kr / dd) * so2_new
                sdo = (kr / dd) * so2_old
                if (so2_new .lt. so2_ptresh) then
                   so4_new = so4_old * expso4
                 else
                   so4_new = sdo * (expso2 - expso4) + so4_old * expso4
                endif
              else
                so4_new = (so4_old + kr * so2_old * dt) * expso4
              endif
              br_pmass(ibranch,itraj,2) = so4_new			! updates % of SO4 mass

c	compute sigma-h (m)

              sh = br_sh(ibranch,itraj)
              ib = nint((blat - ymin_c) * inv_dy_c) + 1 	! lat grid index of puff centre

c	compute incremental distance and update total travel distance (m)

                dx = (blon - br_lon_old(ibranch,itraj)) * coslat(ib)
                dy = (blat - br_lat_old(ibranch,itraj)) 
                deltax = sqrt(dx*dx + dy*dy) * deg2m
                br_totx(ibranch,itraj) = br_totx(ibranch,itraj) + deltax
                
                if(uwind(month,day,hour).ne.0.) then
                   uu = uwind(month,day,hour)
                   vv = vwind(month,day,hour)
                   u = sqrt (uu**2 + vv**2)
                endif
c                print *, month, day, hour, uu, vv, u

                call CSTAB (blon, blat, julday, real(hour), u, p, istab)
                call SIGMA_Y (sh, br_totx(ibranch,itraj), deltax,
     +                        iurbrur, istab, dt)

              br_sh(ibranch,itraj) = sh               ! store computed sigma-h

c	compute puff radius (3*sigma, both directions) 
c	in terms of # of grid cells of conc. grid

              prad = 3. * sh
              iradius = nint(prad * inv_dx_c_m / coslat(ib))
              jradius = nint(prad * inv_dy_c_m)

c	compute indexes of conc. & dep. grid cells to be scanned for current puff:

c	... 'absolute' ones (for dep. normalization - 'boundaries effect')

              ii1 = ipuff - iradius
              ii2 = ipuff + iradius
              jj1 = jpuff - jradius
              jj2 = jpuff + jradius

c	... 'actual' ones of conc. & dep. grid

              i1 = max(ii1, 1)
              i2 = min(ii2, nx_c)
              j1 = max(jj1, 1)
              j2 = min(jj2, ny_c)

c	compute dispersion distance factors for mass

              sum = 0.
              sumtrash = 0.
              
              do j = jj1,jj2			! for each cell of conc. subgrid
                do i = ii1,ii2
                  x = real(i-ipuff) * dx_c * coslat(ib)
                  y = real(j-jpuff) * dy_c 
                  r = sqrt(x*x + y*y) * deg2m	! dist. (puff centre)-(gridpoint) (m)
                  ie = nint((r / sh) * 10.) + 1	! index of tabulated dist. exp term
                  if (ie .le. 41) then		! contribution only if not too far
                    eie = ex2(ie)
                  else
                    eie = 0.
                  endif
                  if (i .ge. i1 .and. i .le. i2 .and.
     +                j .ge. j1 .and. j .le. j2) then	! inside target grid
                    distrib(i,j) = eie
                    ndistrib(i,j) = eie
                    sum = sum + eie
                  else					! outside
                    sumtrash = sumtrash + eie
                  endif
                enddo
              enddo

c	normalize dep. mass distribution factors on subgrid

              sumtot = sum + sumtrash
              if (sumtot .gt. 0.) then
                invsum = 1. / sumtot
                do j = j1,j2
                  do i = i1,i2
                    ndistrib(i,j) = ndistrib(i,j) * invsum
                  enddo
                enddo
              endif

c	compute conc. and dry dep. contributions 
c	(only for layers in contact with terrrain):

              if (reachsrf) then	

c	... compute conc. at puff center in micrograms/m3

                c_vol = emi100th /
     +                  (gfact * sh * sh * z)
                concso2 = br_pmass(ibranch,itraj,1) * c_vol
                concso4 = br_pmass(ibranch,itraj,2) * c_vol * 1.5

c	... update grid concentrations at gridpoints

                do j = j1,j2
                  do i = i1,i2
                    eie = distrib(i,j)
          c_fields(i,j,1,month) = c_fields(i,j,1,month) + concso2 * eie	! (ug/m3)
          c_fields(i,j,2,month) = c_fields(i,j,2,month) + concso4 * eie	! (ug/m3)
                  enddo
                enddo

              endif			! reach surface

           
            endif				! branch inside target grid

          endif				! active branch

        enddo			! branches loop

      enddo		! trajs loop

      return                                                                    
      end                                                                       



      subroutine CONC_AVERAGE
c------------------------------------------------------------------------------
c S. Guttikunda, November 2009
c
c PURPOSE: Update current integration/averaging time interval (hr).
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'

      integer i, j, im, l
      real divn
      

      do im = 1,12
         divn = 1. / navgstep(im)
         do j = 1, ny_c
            do i= 1, nx_c
               do l = 1, nspec
                  c_fields(i, j, l, im) = c_fields(i, j, l, im) * divn
               enddo
            enddo
         enddo
      enddo

      do im = monthb, monthe
         do j = 1, ny_c
            do i = 1, nx_c
               do l = 1, nspec
                  conc_avg(i,j,l) = 
     +         conc_avg(i,j,l) + c_fields(i,j,l,im) / (monthe-monthb+1)
               enddo
            enddo
         enddo
      enddo
      
      return
      end




      subroutine RESET_CONC
c------------------------------------------------------------------------------
c S. Guttikunda, November 2009
c
c PURPOSE: Resets current integration/averaging time interval (hr).
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'

      integer i, j, k, l
      
      dtinteg = 0       ! averaging / integration interval (hr)

      do k = 1,12
         navgstep(k) = 0       ! # of averaging steps
         do i = 1, nx_c
            do j= 1,ny_c
               do l = 1,2
                  c_fields(i,j,l,k) = 0.
                  conc_avg(i,j,l) = 0.
               enddo
            enddo
         enddo
      enddo

      return
      end      









       subroutine BRANCH 
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c Adapted from ATMOS 1.0 model (Arndt & Carmichael, 95), 
c based on BAT model (Heffter, 83)
c
c PURPOSE: Branches current puff according to layers height variations 
c          and day / night transitions
c
c LAST REV.:
c 25.1.99	GC  implicit none
c 27.8.99	GC  reviewed to avoid all loss mechanisms when the
c                   max # of branches per traj is reached; also added 
c                   search for empty slots in branches lists
c 24.11.99      GC  nocturnal BL over urban areas
c 2.3.2000	GC  'vectorized' species
c
c INPUT: via common areas
c
c OUTPUT: via common areas
c
c CALLS: CHECK_MASS, NEW_BRANCH, BR_NEW_PLACE
c------------------------------------------------------------------------------
      implicit none
      
      include 'atmos40.inc'

      integer br_new_place

      integer itraj, nbrnch, ibr, itrans, ipuff, jpuff, iurbrur, nsp
      integer ilon, ilat, l
      integer iplace1, iplace2, iplace3
      real mass_old(nspec), mass_new(nspec), blt_old, blt_new,
     +     factor1, factor2, factor3, ssltop
      logical srise, sset
            


c      include 'check_br_blt.inc'

      nsp = nspec
      
      do itraj = 1, maxtraj               ! for each traj

        nbrnch = nbranches(itraj)         ! current # of branches

        do ibr = 1, nbranches(itraj)              ! for each branch of current traj

          if (br_layer(ibr,itraj) .ne. 0) then      ! if current branch is active 

      ilon = nint(br_lon(ibr,itraj))
      ilat = nint(br_lat(ibr,itraj))

      srise = sunrise(ilon,ilat+90)
      sset = sunset(ilon,ilat+90)

      do l = 1,nspec
        mass_old(l) = br_pmass(ibr,itraj,l)
      enddo
      blt_old = br_blt_old(ibr,itraj)           ! b.l. top at beginning of segment
      blt_new = br_blt(ibr,itraj)               ! b.l. top at end of segment

c       go to different cases according to current layer and day / night

c	adopted layer indexes:

c       1 = surface (night)
c       2 = boundary (day)
c       3 = upper (day)
c       4 = boundary (night)
c       5 = upper (night)
c       6 = surface (day)

      factor1 = 0.
      factor2 = 0.
      factor3 = 0.

      goto (100,200,300,400,500,600), br_layer(ibr,itraj)

c---------------------------------
c ***** current is sfc(nite) *****
c---------------------------------

  100 if (srise) then		! 'transition to day' case
c	WRITE(*,*) 'sfc(nite)'

        br_layer(ibr,itraj) = 2        ! sfc(nite) becomes bndry(day)
c        br_trans(ibr,itraj) = 3        ! store transition index for urban correction

      else                                      ! sfc(nite) remains sfc(nite)

      endif
      goto 1000

c----------------------------------
c ***** current is bndry(day) *****
c----------------------------------

  200 if (sset) then		! day -> night transition cases
c	WRITE(*,*) 'bndry(day)'

        if ((blt_old - blt_new) .gt. dzmin_br) then

c --- b.l. shrinks more than 300 m : generates also upper layer branch
c --- branch: bndry(day) to bndry(nite), sfc(nite), and upper(nite)

          iplace1 = BR_NEW_PLACE (itraj, ibr, 0)
          iplace3 = BR_NEW_PLACE (itraj, ibr, iplace1)
          if (iplace1 .ne. 0) then 
            ssltop = sltop
              ipuff = nint((br_lon(ibr,itraj) - xmin_c) / dx_c) + 1
              jpuff = nint((br_lat(ibr,itraj) - ymin_c) / dy_c) + 1
              iurbrur = 1
              if (iurbrur .eq. 1) ssltop = nultop
            if (iplace3 .ne. 0) then			! generate 3 branches
              factor2 = (blt_new - ssltop) / blt_old
              factor1 = ssltop / blt_old
              factor3 = (blt_old - blt_new) / blt_old
            else 					! generate 2 branches
              factor2 = (blt_new - ssltop) / blt_new
              factor1 = ssltop / blt_new
              factor3 = 0.
            endif
          else						! keep just bndry
            factor2 = 1.
            factor1 = 0.
            factor3 = 0.
          endif

c --- updates current: bndry(day) becomes bndry(nite)

          br_layer(ibr,itraj) = 4
          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor2
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new sfc(nite)                                                                    

          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor1
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace1, 
     +                     1, blt_new, nsp, mass_new, 1)

c --- generates new upper(nite)

          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor3
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace3, 
     +                     5, blt_new, nsp, mass_new, 1)

        else

c --- branch: bndry(day) to bndry(nite) and sfc(nite)

          iplace1 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace1 .ne. 0) then 			! generates 2 branches
            ssltop = sltop
              ipuff = nint((br_lon(ibr,itraj) - xmin_c) / dx_c) + 1
              jpuff = nint((br_lat(ibr,itraj) - ymin_c) / dy_c) + 1
              iurbrur = 1
              if (iurbrur .eq. 1) ssltop = nultop
            factor2 = (blt_old - ssltop) / blt_old
            factor1 = ssltop / blt_old
          else						! keep just bndry
            factor2 = 1.
            factor1 = 0.
          endif

c --- bndry(day) becomes bndry(nite)

          br_layer(ibr,itraj) = 4
          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor2
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new sfc(nite)

          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor1
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace1, 
     +                     1, blt_new, nsp, mass_new, 1)

        endif

      else				! day -> day transition cases

        if ((blt_old - blt_new) .gt. dzmin_br) then

c --- b.l. shrinks more than 300 m : generates also upper layer branch
c --- branch: bndry(day) to bndry(day) and upper(day)

          iplace3 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace3 .ne. 0) then 			! generate 2 branches
            factor2 = blt_new / blt_old
            factor3 = (blt_old - blt_new) / blt_old
          else						! keep just bndry
            factor2 = 1.
            factor3 = 0.
          endif

c --- updates bndry(day)                                                                   

          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor2		! update mass % of 'old' branch
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new upp(day)

          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor3			! compute mass % of 'new' branch
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace3, 
     +                     3, blt_new, nsp, mass_new, 1)

        else

c --- bndry(day) to bndry(day) (no transition: remains the same)

        endif

      endif

      goto 1000

c----------------------------------
c ***** current is upper(day) *****
c----------------------------------

  300 if (sset) then		! day -> night transition cases
c	WRITE(*,*) 'upper(day)'

        if ((blt_new - blt_old) .gt. dzmin_br) then

c --- b.l. rises more than 300 m :
c computes also loss to free trop NO
c --- branch: upper(day) to upper(nite) and bndry(nite)

          iplace2 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace2 .ne. 0) then 			! generate 2 branches
            factor3 = (ultop - blt_new) / (ultop - blt_old)
            factor2 = (blt_new - blt_old) / (ultop - blt_old)
          else						! keep just upper
            factor3 = 1.
            factor2 = 0.
          endif

c --- updates upper(nite)

          br_layer(ibr,itraj) = 5
          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor3 
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new bndry(nite)

          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor2
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace2, 
     +                     4, blt_new, nsp, mass_new, 1)

        else

          br_layer(ibr,itraj) = 5		! upper(day) becomes upper(nite)

        endif

      else					! (day -> day transition cases)

      endif
      goto 1000
                                                
c-----------------------------------
c ***** current is bndry(nite) *****
c-----------------------------------

  400 if (srise) then		! night -> day transition cases
c	WRITE(*,*) 'bndry(nite)'

        if ((blt_old - blt_new) .gt. dzmin_br) then

c --- b.l. shrinks more than 300 m : generates also upper layer branch
c --- branch: bndry(nite) to bndry(day) and upper(day)

          iplace3 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace3 .ne. 0) then 			! generate 2 branches
            ssltop = sltop
              ipuff = nint((br_lon(ibr,itraj) - xmin_c) / dx_c) + 1
              jpuff = nint((br_lat(ibr,itraj) - ymin_c) / dy_c) + 1
              iurbrur = 1
              if (iurbrur .eq. 1) ssltop = nultop
            factor2 = (blt_new - ssltop) / (blt_old - ssltop)
            factor3 = (blt_old - blt_new) / (blt_old - ssltop)
          else						! keep just bndry
            factor2 = 1.
            factor3 = 0.
          endif

c --- bndry(nite) becomes bndry(day)                                                    

          br_layer(ibr,itraj) = 2
          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor2
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new upper(nite)
                                                                 
          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor3
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace3, 
     +                     3, blt_new, nsp, mass_new, 1)

        else

c --- bndry(nite) becomes bndry(day)                                                    

          br_layer(ibr,itraj) = 2

        endif

      else						! night -> night transition cases

        if ((blt_old - blt_new) .gt. dzmin_br) then

c --- b.l. shrinks more than 300 m : generates also upper layer branch
c --- branch: bndry(nite) to bndry(nite) and upper(nite)

c --- updates bndry(nite)                                                                  

          iplace3 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace3 .ne. 0) then 			! generate 2 branches
            ssltop = sltop
              ipuff = nint((br_lon(ibr,itraj) - xmin_c) / dx_c) + 1
              jpuff = nint((br_lat(ibr,itraj) - ymin_c) / dy_c) + 1
              iurbrur = 1
              if (iurbrur .eq. 1) ssltop = nultop
            factor2 = (blt_new - ssltop) / (blt_old - ssltop)
            factor3 = (blt_old - blt_new) / (blt_old - ssltop)
          else						! keep just bndry
            factor2 = 1.
            factor3 = 0.
          endif

          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor2
          enddo
          call CHECK_MASS (ibr,itraj)

c -- generates upper(nite)                                                              
    
          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor3
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace3, 
     +                     5, blt_new, nsp, mass_new, 1)

        else

c --- bndry(nite) to bndry(nite) (no transition: remains the same)

        endif

      endif
      goto 1000

c-----------------------------------
c ***** current is upper(nite) *****
c-----------------------------------

  500 if (srise) then			! night -> day transition cases
c	WRITE(*,*) 'upper(nite)'

        if ((blt_new - blt_old) .gt. dzmin_br) then

c --- b.l. rises more than 300 m :
c --- branch: upper(nite) to upper(day) and bndry(day)

          iplace2 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace2 .ne. 0) then 			! generate 2 branches
            factor3 = (ultop - blt_new) / (ultop - blt_old)
            factor2 = (blt_new - blt_old) / (ultop - blt_old)
          else						! keep just upper
            factor3 = 1.
            factor2 = 0.
          endif

c --- upper(nite) becomes upper(day)

          br_layer(ibr,itraj) = 3             
          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor3
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new bndry(day)                                                                   

          itrans = 2        ! store transition index for urban correction
          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor2
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace2, 
     +                     2, blt_new, nsp, mass_new, itrans)

        else    

c --- upper(nite) to upper(day)

          br_layer(ibr,itraj) = 3

        endif

      else							! night -> night transition cases

        if ((blt_new - blt_old) .gt. dzmin_br) then

c --- b.l. rises more than 300 m :
c --- branch: upper(nite) to upper(nite) and bndry(nite)

          iplace2 = BR_NEW_PLACE (itraj, ibr, 0)
          if (iplace2 .ne. 0) then 			! generate 2 branches
            factor3 = (ultop - blt_new) / (ultop - blt_old)
            factor2 = (blt_new - blt_old) / (ultop - blt_old)
          else						! keep just upper
            factor3 = 1.
            factor2 = 0.
          endif

c --- updates upper(nite)                                                                  

          do l = 1,nspec
            br_pmass(ibr,itraj,l) = mass_old(l) * factor3
          enddo
          call CHECK_MASS (ibr,itraj)

c --- generates new bndry(nite)                                                                  

          do l = 1,nspec
            mass_new(l) = mass_old(l) * factor2
          enddo
          call NEW_BRANCH (itraj, ibr, nbrnch, iplace2, 
     +                     4, blt_new, nsp, mass_new, 1)
   
        else

c --- upper(nite) to upper(nite): no transition: remains the same

        endif

      endif
      goto 1000

c--------------------------------
c ***** current is sfc(day) ***** (just to trap errors)
c--------------------------------
                                 
  600 continue          ! write(*,*) 'Warning: a sfc(day) occurred!'
c	WRITE(*,*) 'sfc(day)'

      if (sset) then

c --- 'transition to night' case: sfc(day) becomes sfc(nite)

        br_layer(ibr,itraj) = 1                                                                

      else

c --- sfc(day) becomes bndry(day)

        br_layer(ibr,itraj) = 2 

      endif
      goto 1000

c      include 'urban_branching.inc'

 1000       continue

          endif

        enddo                   ! next branch

        nbranches(itraj) = nbrnch          ! updates # of branches

      enddo             ! next traj

c      include 'layers_debug.inc'	! debug

      return
      end                                                                       









      subroutine CHECK_MASS (ibranch, itraj)
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c
c PURPOSE: Check masses of a given branch of a given traj: if lower than
c          threshold(s), reset branch coordinates
c
c LAST REV.:
c 25.1.99	GC  implicit none
c 2.3.2000	GC  'vectorized' species
c
c INPUT:
c ibranch	I  index of branch to be checked
c itraj		I  index of traj to be checked
c other vars via common areas
c
c OUTPUT: via common areas
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'

      integer ibranch, itraj

      integer l
      logical killit      
      

      killit = .true.
      do l = 1,nspec
        if (br_pmass(ibranch,itraj,l) .ge. min_mass) killit = .false.
      enddo
      
      if (killit)	! terminates current branch seting layer index to 0 
     +   br_layer(ibranch,itraj) = 0

      return
      end







      subroutine NEW_BRANCH (itraj, ibranch, nbrnch,
     +                       iplace, inewlayer, blt_new,
     +                       nsp, mass_new, itrans)
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c
c PURPOSE: Generates a new branch in a given layer from a given branch 
c          of a given traj; no new branch if masses < threshold(s)
c
c LAST REV.:
c 25.1.99	GC  implicit none
c 27.8.99	GC  added index of empty slot to place the new branch
c 2.3.2000	GC  'vectorized' species
c
c INPUT: 
c itraj		I  index of current traj
c ibranch	I  index of current branch
c inewlayer	I  index of layer of new branch
c iplace	I  index of insertion point for new branch
c blt_new	R  b.l. height of new branch
c nsp		I  # of species
c mass_new	R  mass % of new branch
c itrans	I  layer transition index
c other vars via common areas
c
c OUTPUT: via common areas
c
c INPUT/OUTPUT:
c nbrnch	I  # of branches of current traj
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
      
      include 'atmos40.inc'

      integer itraj, ibranch, nbrnch, iplace, itrans, inewlayer, nsp
      real blt_new, mass_new(nsp)

      integer l
      logical addit
      


      addit = .false.
      do l = 1,nspec
        if (mass_new(l) .gt. min_mass) addit = .true.	! check mass % of before branch insertion
      enddo
      
      if (addit) then

        br_lat(iplace,itraj) = br_lat(ibranch,itraj)    ! lat of new branch
        br_lon(iplace,itraj) = br_lon(ibranch,itraj)    ! lon of new branch
        br_lat_old(iplace,itraj) = br_lat_old(ibranch,itraj)    ! (old) lat of new branch
        br_lon_old(iplace,itraj) = br_lon_old(ibranch,itraj)    ! (old) lon of new branch
        br_layer(iplace,itraj) = inewlayer              ! layer of new branch
        br_blt(iplace,itraj) = blt_new                  ! b.l. height of new branch
        do l = 1,nspec
          br_pmass(iplace,itraj,l) = mass_new(l)	! mass % of new branch
        enddo
        br_totx(iplace,itraj) = br_totx(ibranch,itraj)  ! total travel distance
        br_sh(iplace,itraj) = br_sh(ibranch,itraj)      ! sigma-h
        
        if (iplace .gt. nbranches(itraj)) then
          nbrnch = nbrnch + 1			! increments branch counter of current traj
        endif

      endif

      return
      end







      integer function BR_NEW_PLACE (itraj, ibranch, iforbidden)
c------------------------------------------------------------------------------
c G. Calori, 27.8.99
c
c PURPOSE: Check in data arrays the existence of a suitable insertion point
c          for a new branch
c
c LAST REV.:
c
c INPUT: 
c itraj		I  index of current traj
c ibranch	I  index of current branch
c iforbidden	I  index of a forbidden place for a new branch
c other vars via common areas
c
c OUTPUT: index of empty slot for the new branch; 0 otherwise
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
      
      include 'atmos40.inc'

      integer itraj, ibranch, iforbidden




      br_new_place = 1
      do while (br_new_place .le. maxbr .and.
     +          (br_layer(br_new_place,itraj) .ne. 0 .or.
     +           br_new_place .eq. iforbidden)) 
        br_new_place = br_new_place + 1
      enddo
      if (br_new_place .gt. maxbr) br_new_place = 0
c	WRITE(*,*) 'itraj, ibranch, iforbidden, new_place = ',
c     +    BR_NEW_PLACE

      return
      end



      subroutine READ_METEO
c------------------------------------------------------------------------------
c S. Guttikunda, December 2009
c
c PURPOSE: Read a new set of meteorological fields
c Converts the wind speed and wind direction into u and v fields
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'
 
      integer it, jt, kt, im, id, ih, icount
      real ws, wd, hm, pr, piwd

      write(io_msg,'(/,a)') '--- Reading Meteorological data',
     +   ' '  

      do it = 1, 12
      do jt = 1, 31
      do kt = 1, 24
          uwind(it, jt, kt) = 0.0
          vwind(it, jt, kt) = 0.0
          hmix(it, jt, kt) = 0.0
          precip(it, jt, kt) = 0.0
      enddo
      enddo
      enddo
      
      icount = 0

      open(22,file='meteofile',status='old')

  10  read(22,*,end=40) im, id, ih, ws, wd, hm, pr
        piwd = wd/180.*22./7.
        uwind(im,id,ih) = (-1)*ws*sin(piwd) 
        vwind(im,id,ih) = (-1)*ws*cos(piwd)
        hmix(im,id,ih) = hm
        precip(im,id,ih) = pr
      icount = icount + 1
      go to 10
      
  40  write(io_msg,*) '--- Number of met points entered = ', icount

      if(uwind(1,1,dttransp).eq.0.) uwind(1,1,dttransp) = 0.1
      if(vwind(1,1,dttransp).eq.0.) vwind(1,1,dttransp) = 0.1
      if(hmix(1,1,dttransp).eq.0.) hmix(1,1,dttransp) = 100.
     
      return
      end

      
      
      
      
            
      subroutine SET_HMIX 
c------------------------------------------------------------------------------
c G. Calori, 25.1.99
c
c PURPOSE: For each branch of each traj assigns new b.l. height, 
c          by linear interpolation of input field
c------------------------------------------------------------------------------
      implicit none
                                                                                
      include 'atmos40.inc' 

      integer itraj, ib
 
      do itraj = 1,maxtraj		! for each traj 
 
        do ib = 1,nbranches(itraj)		! for each branch 
 
          if (br_layer(ib,itraj) .ne. 0) then      ! if current branch is active 
 
            br_blt_old(ib,itraj) = br_blt(ib,itraj)      ! store old mixing height 

            br_blt(ib,itraj) = hmix(month,day,hour)

            br_blt(ib,itraj) = nint(max(real(br_blt(ib,itraj)),		! m.l. must be > s.l.
     +                                  sltop))
 
          endif 
 
        enddo           ! next branch 
  
      enddo		! next traj 
 
      return                                                                     
      end                                                                        
 

 
 
 
 
      subroutine SIGMA_H (sh)
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c
c PURPOSE: Computes horizontal sigma-h, basing on fly time (Heffter, '83)
c------------------------------------------------------------------------------
      implicit none

      include 'atmos40.inc'

      real sh


      sh = sh + lr_sh_param

      return
      end






      subroutine SIGMA_Y (sigmay, xtravel, deltax, iurbrur, 
     +                    istab, deltat)
c------------------------------------------------------------------------------
c G. Calori, Mar '97
c
c PURPOSE: Returns Gaussian plume sigma-y (m) at a given downwind distance
c
c Short range:
c - Pasquill-Gifford-Turner(PGT) coeffs. for rural conditions, use
c   values of coeffs. and exponents from MESOPUFF II approximations.
c - Briggs urban coeffs. for each PGT class for urban conditions.
c   Computed exactly as in ISC6-8.
c Long range:
c Heffter time dependent growth coeffs.
c
c LAST REV.:
c 9.3.99  GC	implicit none
c
c INPUT:
c sigmay  	R  actual sigma-y (m)
c xtravel 	R  total downwind travel distance (m)
c deltax  	R  incremental travel distance (m)
c iurbrur 	I  urban/rural flag: 0 = rural; 1 = urban
c istab   	I  PGT stability index (1-6)
c deltat  	R  incremental travel time (s), since 'actual sigma-y' calc.
c
c OUTPUT:
c sigmay  	R  updated sigma-y (m)
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none

      real sigmay, xtravel, deltax, deltat
      integer iurbrur, istab
      
      real xlrt_sigma
      parameter (xlrt_sigma = 30000.)    ! short-/long-range sigmay switching distance (m) (CALPUFF = 10 km)

      real xvirt, xtot, a2, bsyold, sigmay_old
      real ay_pgt(6), by_pgt(6), ray_pgt(6), rby_pgt(6), 
     +     ay_burb(6), by_burb(6)
      
c     Pasquill-Gifford-Turner(PGT) sigma-y coeffs. and exponents from MESOPUFF II approximations

      data ay_pgt  / 0.36,  0.25,  0.19,  0.13,  0.096,   0.063/  ! PGT Y coeffs. for each PGT class
      data by_pgt  /  0.9,   0.9,   0.9,   0.9,    0.9,     0.9/  ! PGT Y exponents for each PGT class
      data ray_pgt /2.778, 4.000, 5.263, 7.692, 10.417,  15.873/  ! reciprocals of AY_PGT(6)
      data rby_pgt /1.111, 1.111, 1.111, 1.111,  1.111,   1.111/  ! reciprocals of BY_PGT(6)

c     Briggs-urban sigma-y sigmay coeffs. as in ISC

      data ay_burb /  0.32,   0.32,   0.22,   0.16,   0.11,   0.11/    ! urban Y coeffs. for each PGT class
      data by_burb /0.0004, 0.0004, 0.0004, 0.0004, 0.0004, 0.0004/    ! reciprocal length scale (1/m) for each PGT class

c      data sigmin/0.01/

                          
                          
      sigmay_old = sigmay
      
      if (xtravel .lt. xlrt_sigma) then       ! short-range

        if (iurbrur .eq. 0) then           ! rural conditions
      
c     compute Pasquill-Gifford-Turner(PGT) sigma-y (rural conditions)

          xvirt = (sigmay_old * ray_pgt(istab)) ** rby_pgt(istab) ! virtual distance (m)
          xtot = xvirt + deltax                                   ! total distance (m)
          sigmay = ay_pgt(istab) * xtot ** by_pgt(istab)

        else                    ! urban conditions

c     compute Briggs-urban sigma-y

c     virtual distance (m) corresponding to sigmay_old

          a2 = ay_burb(istab)**2
          bsyold = 0.5 * by_burb(istab) * sigmay_old
          xvirt = sigmay_old * (bsyold + sqrt(bsyold**2 + a2)) / a2

        xtot = xvirt + deltax                                   ! total distance (m)
        sigmay = ay_burb(istab) * xtot / sqrt(1. + by_burb(istab)*xtot)

        endif

      else                                    ! long-range

c     compute Heffter (1965) sigmay  

        sigmay = sigmay_old + 0.5 * deltat
        
      endif

      return
      end






      subroutine CSTAB (lon, lat, julday, xhour, u, p, istab)
c------------------------------------------------------------------------------
c G. Calori, Mar '97
c
c PURPOSE: Returns PGT stability class
c
c LAST REV.:
c 9.3.99  GC	implicit none
c
c INPUT:
c lon, lat	R  geog. coordinates
c julday	I  Julian day
c xhour		R  hour of day
c u		R  ground level wind speed (m/s)
c p		R  precipitation rate (mm/h)
c
c OUTPUT:
c istab   	I  PGT stability index (1-6)
c
c CALLS: SUN_RAD
c------------------------------------------------------------------------------
      implicit none

      real lon, lat, xhour, u, p
      integer julday, istab
      
      real srad
      integer iu, irad
      integer d_stab_tab(4,5), n_stab_tab(3,5)

      data d_stab_tab /1, 1, 2, 3, 3,
     +                 1, 2, 2, 3, 4,
     +                 2, 3, 3, 4, 4,
     +                 2, 3, 3, 4, 4/
      data n_stab_tab /6, 5, 4, 4, 4,
     +                 6, 6, 5, 4, 4,
     +                 6, 6, 5, 4, 4/

      
      call SUN_RAD (lon, lat, julday, xhour, 0, srad)

      if (u .le. 2.) then
        iu = 1
      else if (u .le. 3.) then
        iu = 2
      else if (u .le. 5.) then
        iu = 3
      else if (u .le. 6.) then
        iu = 4
      else
        iu = 5
      endif

      if (srad .gt. 0.) then          ! day
        if (srad .ge. 700.) then
          irad = 1
        else if (srad .ge. 350.) then
          irad = 2
        else
          irad = 3
        endif
        if (p .gt. 0.) irad = irad + 1    ! for the time being, use p as surrogate of tcc
        istab = d_stab_tab(irad,iu)
      else                            ! night
        irad = 1
        if (p .gt. 0.) irad = irad + 1    ! for the time being, use p as surrogate of tcc
        istab = n_stab_tab(irad,iu)
      endif

      return
      end          
      






      subroutine SUN_RAD (lon, lat, julday, xhour, icloud, solar_rad)
c------------------------------------------------------------------------------
c G. Calori, Mar '97
c
c PURPOSE: Total solar radiation from lon, lat, 
c          day of year & hour, total cloud cover
c
c LAST REV.:
c 9.3.99  GC	implicit none
c
c INPUT:
c lon, lat	R  geog. coordinates
c julday	I  Julian day
c xhour		R  hour of day (UTC)
c icloud	I  total cloud cover (0,1..8)
c
c OUTPUT:
c solar_rad	R  total solar radiation (W/m2)
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none

      real lon, lat, xhour, solar_rad
      integer julday, icloud

      real pi, alfa, d2r, h2r, si
      parameter (pi = 3.141592)
      parameter (alfa = 2. * pi /365.25)
      parameter (d2r = pi / 180.)
      parameter (h2r = pi / 12.)
      parameter (si = 1370.) ! solar irradiance (W/m2)
 
      real sol_decl, rlon, rlat, sin_solar_elev, trans
      real cc_high, cc_mid, cc_low
      

      sol_decl = 0.409 * cos(real(julday-173) * alfa)   ! solar declination angle (rad)

      rlon = lon * d2r
      rlat = lat * d2r
      sin_solar_elev = sin(rlat) * sin(sol_decl) -
     +                 cos(rlat) * cos(sol_decl) *
     +                 cos(xhour * h2r + rlon) ! sine of solar elevation angle

      if (sin_solar_elev .gt. 0.) then                    ! day

c     cloud cover fractions

        cc_high = real(icloud) * 0.125	! (high)
        cc_mid = cc_high		! (middle)
        cc_low = cc_high		! (low)

c     sky transmissivity

        trans = (0.6 + 0.2 * sin_solar_elev) *
     +    (1 - 0.4 * cc_high) * (1 - 0.7 * cc_mid) *
     +    (1 - 0.4 * cc_low)

c     downwelling solar radiation at the surface

        solar_rad = si * trans * sin_solar_elev

      else                                                ! night

        solar_rad = 0.

      endif

      return
      end







      subroutine RESET_TRAJS 
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c
c PURPOSE: Reset parameters of all trajs
c
c LAST REV.:
c 25.1.99	GC  implicit none
c
c INPUT: via common areas
c
c OUTPUT: via common areas
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
 
      include 'atmos40.inc' 

      integer itraj, ibranch
      
                                                                     
      ntrajtot = 0                 ! reset traj counter 
 
      do itraj = 1,maxtraj 
        do ibranch = 1,maxbr 
          br_layer(ibranch,itraj) = 0 
          nbranches(itraj) = 0 
        enddo 
      enddo 
 
      return                                                                     
      end                                                                        
 
 
 
 
 
 
 
      subroutine START_TRAJS
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c Freely adapted from ATMOS 1.0 model (Arndt & Carmichael, 95), 
c based on BAT model (Heffter, 83)
c
c PURPOSE: Starts a new set trajectories from all current sources
c          at current simulation time
c
c LAST REV.:
c 26.1.99	GC  implicit none
c 2.3.2000	GC  'vectorized' species
c
c INPUT: via common areas
c
c OUTPUT: via common areas
c
c CALLS: SLINT2D
c------------------------------------------------------------------------------
      implicit none
 
      include 'atmos40.inc' 
 
      real slint2d
 
      integer iplace, ib, l
      real blon, blat
      logical iisday, found 

c --- generate new trajs according to specified sources time resolution

        if (mod(hour, dttransp) .eq. 0) then
                
c --- search a place in arrays for the new traj  
 
          iplace = 0 
          found = .false. 
          do while (.not. found .and. iplace .lt. maxtraj) 
            iplace = iplace + 1 
            if (nbranches(iplace) .eq. 0) found = .true.
          enddo 
          if (.not. found) then 
            write(*,*) '*** START_TRAJS *** ',
     +        'Max # of trajs exceeded. Max allowed = ', maxtraj 
            stop 
          endif       
 
          life_traj(iplace) = 0  		! traj life (hr)
          traj_hte(iplace) = ihte	! source height flag 
 
          do ib = 1,maxbr           ! initialize all branches of current traj 
            br_lat(ib,iplace) = xmissing
            br_lon(ib,iplace) = xmissing 
            br_layer(ib,iplace) = 0 
            br_blt(ib,iplace) = 0 
            br_blt_old(ib,iplace) = 0
            do l = 1,nspec 
              br_pmass(ib,iplace,l) = 0.
            enddo
            br_totx(ib,iplace) = 0.
            br_sh(ib,iplace) = 0.
          enddo 
 
c --- set parameters of unique branch 
                                                                                
          nbranches(iplace) = 1		! traj just started: 1st branch 
          traj_emiss(iplace) = emiss(imatrix) 
          blon = source_lon(imatrix)
          blat = source_lat(imatrix)
          br_lat(1,iplace) = blat
          br_lon(1,iplace) = blon
          br_lat_old(1,iplace) = source_lat(imatrix) 
          br_lon_old(1,iplace) = source_lon(imatrix) 
 
c --- assign correct layer according to day/night, source height and sector 

          iisday = isday(nint(blon), nint(blat)+90)	! day/night flag
 
c --- adopted layer indexes: 
c       1 = surface (night) 
c       2 = boundary (day) 
c       3 = upper (day) 
c       4 = boundary (night) 
c       5 = upper (night) 
c       6 = surface (day) 

          if (iisday) then		! day cases 
 
              if (ihte .eq. 0 .or. ihte .eq. 1) then
                br_layer(1,iplace) = 2			! in b.l. 
              else if (ihte .eq. 2) then
                br_layer(1,iplace) = 3			! volc. in u.l.
              endif
  
          else				! night cases 
 
            if (ihte .eq. 0) then 
              br_layer(1,iplace) = 1			! in surface l. 
            else if (ihte .eq. 1) then
              br_layer(1,iplace) = 4			! elev. source: in b.l. 
            else if (ihte .eq. 2) then
              br_layer(1,iplace) = 5			! volc. in u.l. 
            endif 
 
          endif 
 
c --- assign initial b.l. height 
 
          br_blt(1,iplace) = hmix(month, day, hour)

          br_blt(1,iplace) = nint(max(real(br_blt(1,iplace)), sltop))	! m.l. must be > s.l.

          br_blt_old(1,iplace) = br_blt(1,iplace)
 
c --- species mass fractions at emission
 
          do l = 1,nspec
            br_pmass(1,iplace,l) = spec_emiperc(l)
          enddo
 
c --- initial sigma-h

          if (ihte.eq.1) then       ! elevated sources
            br_sh(1,iplace) = sh0_lps
          else                      ! area sources
            br_sh(1,iplace) = sh0_diff
          endif

        endif
        
      return                                                                     
      end                                                                        
 
 


 
 
 
      subroutine TERM_TRAJ 
c------------------------------------------------------------------------------
c G. Calori, Jan 95
c
c PURPOSE: Terminates all trajs older than 5 days, 
c          by setting to 0 their # of branches 
c
c LAST REV.:
c 26.1.99	GC  implicit none
c
c INPUT: via common areas
c
c OUTPUT: via common areas
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
 
      include 'atmos40.inc' 

      integer itraj

 
      do itraj = 1,maxtraj
        if (nbranches(itraj) .gt. 0) then 
          if (life_traj(itraj) .gt. trajdur) then
            nbranches(itraj) = 0 		! kill all branches
          endif
        endif 
      enddo 
 
      return 
      end 
 

 
 
 

      subroutine SET_DN_FLAGS
c------------------------------------------------------------------------------
c G. Calori, Jan 99
c
c PURPOSE: Set day/night transition flags for current day
c          for all points of the domain (wind)
c
c LAST REV.:
c
c INPUT: via common areas
c
c OUTPUT: via common areas
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
 
      include 'atmos40.inc' 

      integer lonmin, lonmax, latmin, latmax, ii
      integer ilon, ilat, iilat
      real hr_loc, htol, sunrise_hr, sunset_hr
       

c	points are 'integer' lats and lons covering 
c	intersection of wind and conc domains
 
      lonmin = int(xmin_c)
      lonmax = int(xmax_c + 0.5)
      latmin = int(ymin_c)
      latmax = int(ymax_c + 0.5)

      htol = dttransp / 2.	! sunrise/sunset tolerance (hr)

      do ilat = latmin,latmax

        call SUN_RISE_SET (real(ilat), julday, sunrise_hr, sunset_hr)
        iilat = ilat + 90

        do ilon = lonmin,lonmax

c	refer to local hour

          hr_loc = real(hour) + real(ilon) / 15.
          ii = int(hr_loc)
          hr_loc = real(mod(ii,24)) + (hr_loc - real(ii))

          if (hr_loc .le. (sunrise_hr - htol)) then		! night
            sunrise(ilon,iilat) = .false.
            isday(ilon,iilat) = .false.
            sunset(ilon,iilat) = .false.
          else if (hr_loc .le. (sunrise_hr + htol)) then	! night --> day
            sunrise(ilon,iilat) = .true.
            isday(ilon,iilat) = .true.
            sunset(ilon,iilat) = .false.
          else if (hr_loc .le. (sunset_hr - htol)) then		! day
            sunrise(ilon,iilat) = .false.
            isday(ilon,iilat) = .true.
            sunset(ilon,iilat) = .false.
          else if (hr_loc .le. (sunset_hr + htol)) then		! day --> night
            sunrise(ilon,iilat) = .false.
            isday(ilon,iilat) = .false.
            sunset(ilon,iilat) = .true.
          else							! night 
            sunrise(ilon,iilat) = .false.
            isday(ilon,iilat) = .false.
            sunset(ilon,iilat) = .false.
          endif 

        enddo 
      enddo 

      return 
      end 
 
 
 
 



      subroutine SUN_RISE_SET (lat, julday, sunrise_hr, sunset_hr)
c-----------------------------------------------------------------------------
c G. Calori, Jan 99
c
c PURPOSE:  Compute sunrise and sunset hours (UTC) for
c           a given latitude and day of the year
c
c INPUT:
c LAT             R    latitude (degrees)
c JULDAY          I    Julian day
c
c OUTPUT:
c SUNRISE_HR      R    sunrise hour (UTC)
c SUNSET_HR       R    sunset hour (UTC)
c
c CALLS: -
c-----------------------------------------------------------------------------
      implicit none
                                                        
      integer julday
                                  
      real pi, pi2, d2r, r2h
      parameter (pi = 3.14159265359)
      parameter (pi2 = pi * 2.)
      parameter (d2r = pi / 180.)
      parameter (r2h = 12. / pi)

      real lat, latrad
      real sunrise_hr, sunset_hr
      real sol_decl, arg


      latrad = lat * d2r
 
c     solar declination angle

      sol_decl = 0.409 * cos ((real(julday) - 173.) * pi2 / 365.25)	! (rad)
 
      arg = acos (tan(latrad) * tan(sol_decl))
      
      sunrise_hr = arg * r2h
      sunset_hr =  (pi2 - arg) * r2h

      return
      end





            
 
 
 

      integer function STRL(string)
c------------------------------------------------------------------------------
c G. Calori, 10.93
c
c PURPOSE:    Returns lenght of a string, after trimming trailing blanks.
c
c INPUT:
c   string    C*(*)   string to be evaluated
c
c OUTPUT:
c   string lenght
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
      character*(*) string

      strl = len(string)
      do while (strl .ge. 1 .and. string(strl:strl) .eq. ' ')
        strl = strl - 1
      enddo

      return
      end









      subroutine SKIP_COMMENT_LINES (iunit, start_char)
c------------------------------------------------------------------------------
c G. Calori, 14.10.97
c
c LAST REV.: 10.2.99    GC  simply return instead of msg if eof
c
c PURPOSE:    Skip comments lines (starting with 'start_char' in a
c             formatted file.
c
c INPUT:
c   iunit         I   fortran input unit number
c   start_char    C*1 character in 1st column identifying comment lines
c
c OUTPUT: -
c
c CALLS: -
c------------------------------------------------------------------------------
      implicit none
      integer iunit, ios
      character*1 start_char, buffer

      buffer = start_char
      do while (buffer .eq. start_char)
        read (iunit, '(a1)', iostat=ios) buffer
        if (ios .ne. 0) return
      enddo
      backspace (iunit)
      return
      end








       function julian(imon,iday)
c------------------------------------------------------------------------------
c S. Guttikunda, December 2009
c Computes JULIAN day
c------------------------------------------------------------------------------

       implicit none
       include 'atmos40.inc'
       integer im, julian, imon, iday
    
       julday = 0
       
       do im = 1, imon-1
          julday = julday + ndaym(im)
       enddo

       julday = julday + iday

       return
       end


