c------------------------------------------------------------------------------
c ATMOS-2 include file
c
c G.Calori, Jan 99
c S. Guttikunda, December 2009
c
c------------------------------------------------------------------------------

c	sizes of main arrays 
 
      integer maxx_c, maxy_c
      integer maxsrc, max_trajsrc, maxbr, maxtraj
 
      parameter (maxx_c = 35)     	! max # of conc & dep grid cells (lon) 
      parameter (maxy_c = 35)     	! max # of conc & dep grid cells (lat) 
 
      parameter (maxsrc = 1)    	! max # of emitting sources per sector
      parameter (max_trajsrc = 41)	! max # of contemporary traj. for each source - 3-hourly runs
      parameter (maxbr = 32) 		! max number of branches for each traj.

      parameter (maxtraj = max_trajsrc * maxsrc)	! max total # of contemporary traj

c	constants

      real tou, pi, pi2, d2r, rearth, deg2m
      parameter (tou = 3600.)           ! mass balance eq. integration time step (s)
      parameter (pi = 3.14159265359)
      parameter (pi2 = pi * 2.)
      parameter (d2r = pi / 180.)
      parameter (rearth =  6368.)			! earth radius (km)
      parameter (deg2m = pi2 * rearth * 1000. / 360.)	! one deg in m

c	layers params
 
      real sltop, ultop, nultop
      parameter (sltop = 300.)            ! height of layer 1 top (surface layer) 
      parameter (ultop = 6000.)           ! height of layer 3 top (upper layer) 
      parameter (nultop = 500.)           ! height of nocturnal boundary layer over urban areas 
      integer dzmin_br
      parameter (dzmin_br = 300)	! min dz for a new branch

c	i/o units & files
 
      integer
     +  io_ini, io_out, io_msg,
     +  isector, imatrix, ihte, iwrite

      common /io_units/ 
     +  io_ini, io_out, io_msg,
     +  isector, imatrix, ihte, iwrite

      character*256
     + inifile 
 
      common /filenames/ 
     +  inifile
 
c	archives grid & time params
c	(coord. correspond to SW corners of grid cells)
 
c	... concentrations

      integer nx, ny, nx_c, ny_c, ngrids	! # of grid cells 
      real dx_c, dy_c				! grid step sizes
      real ymax_c, ymin_c, xmin_c, xmax_c	! lon & lat min & max
      real ymax_o, ymin_o, xmin_o, xmax_o	! lon & lat min & max
      real xgrid, ygrid				! lon & lat first cell


      common /conc_grid/			! conc & dep grid parameters 
     +  nx_c, ny_c, dx_c, dy_c,
     +  ymin_c, ymax_c, xmin_c, xmax_c,
     +  ymin_o, ymax_o, xmin_o, xmax_o,
     +  nx, ny, ngrids,
     +  xgrid, ygrid
 
c	time related params

      integer trajdur		! traj duration (hr) 
      integer dttransp		! transport time step (hr)
      integer ntransp		! # of transport time steps during whole simul.
      integer itransp		! current transport time step since beginning of simul.
      integer dtout		! output time step (hr)
      integer msave		! last saved month (monthly output)
      integer ysave		! last saved year (yearly output)
      logical safeout		! if true, output files are open/close at each writing cycle
				! if false, they are kept opened
      common /time_param/ 
     +  trajdur, dttransp, ntransp, itransp, 
     +  dtout, msave, ysave, safeout

      integer bdate, btime			! sim. start date & time
      integer dayb, monthb, yearb, hourb	! sim. start date & time
      integer edate, etime			! sim. stop date & time
      integer daye, monthe, yeare, houre	! sim. stop date & time
      integer date3d, time3d			! current date & time during simulation
      integer day, month, year, hour		! current date & time during simulation
      integer julday, ndaym(12)			! current Julian day during simulation 

      common /clock/ 
     +  bdate, btime, dayb, monthb, yearb, hourb,
     +  edate, etime, daye, monthe, yeare, houre,
     +  date3d, time3d, day, month, year, hour, 
     +  julday, ndaym

c	day, sunrise & sunset flags for each lon and lat

      logical isday(360,180)	! at each point, true if currently is day; false if night
      logical sunrise(360,180)	! at each point, true if currently is sunrise; false elsewhere
      logical sunset(360,180)	! at each point, true if currently is sunset; false elsewhere
      common /transitions/
     +  isday, sunrise, sunset

c	topography
 
      real coslat(maxy_c)		! cosine of latitudes at gridpoints of windfields

      common /latcos/ 
     +  coslat

c	meteorological data
c	... winds (m/s)
 
      real uwind(12,31,24), 
     +     vwind(12,31,24),	! avg u and v windfields at each level
     +     hmix(12,31,24),
     +     precip(12,31,24)

      common /meteo/
     +  uwind, vwind, hmix, precip

c	species

      integer nspec, ipoll
      parameter (nspec = 2)		! # of chemical species

      real spec_emiperc(nspec)		! partition of emissions among species (as % of emission)
      real massfact(nspec)		! mass unit conversion factors (from species to dep. units)
      
      common /spec_param/
     +  spec_emiperc, massfact, ipoll
                  
c	sources
 
      real source_lat(maxx_c*maxy_c), 
     +     source_lon(maxx_c*maxy_c),		! sources coordinates 
     +     emiss(maxx_c*maxy_c)		! sources emission strenght (grams/(emission time step))

      common /sources/
     +  source_lat, source_lon, emiss 

c	puffs branches data
 
      integer ntrajtot			! current total # of traj  
      integer nbrtot			! current total # of branches  
      integer nbranches(maxtraj) 	! current # of branches of each traj
      integer life_traj(maxtraj)       	! current life of each traj (hr since emission)
      integer traj_hte(maxtraj)         ! source height flag of each traj (0 = diffuse; 1 = LPS; 2 = volcanoes)
      real traj_emiss(maxtraj)		! emission associated to each traj (grams/(emission time step))
      real br_lat(maxbr,maxtraj), 
     +     br_lon(maxbr,maxtraj)	! current coord. of each traj&branch
      real br_lat_old(maxbr,maxtraj), 
     +     br_lon_old(maxbr,maxtraj)	! coord. of each traj&branch, at beginning of curr transp time step 
      integer br_layer(maxbr,maxtraj)   ! layer index, for each branch and traj 
c      integer br_trans(maxbr,maxtraj)	! transition index, for each branch and traj, for urban corrections 
      integer br_blt(maxbr,maxtraj),   	! current boundary layer top , for each branch and traj 
     +        br_blt_old(maxbr,maxtraj)	! b.l. top at beginning of previous transp time step, for each traj&branch
      real br_pmass(maxbr,maxtraj,nspec)	! mass of each species in each traj&branch (as % of mass initially released)
      real br_totx(maxbr,maxtraj)     	! current branches total travel distances (m)
      real br_sh(maxbr,maxtraj)     	! current branches sigma-h (m)

      common /branches/ 
     +  ntrajtot, nbrtot, nbranches, 
     +  life_traj, traj_hte, traj_emiss, 
     +  br_lat, br_lon, br_lat_old, br_lon_old, 
     +  br_layer, br_blt, br_blt_old,	! br_trans,
     +  br_pmass, br_totx, br_sh

c	diffusion & mass calc. params
 
      real sh0_diff		! initial horizontal sigma for diffuse sources (m)
      real sh0_lps		! initial horizontal sigma for LPS (m)
      real lr_sh_param		! multip. factor for long-range sigma-h
      common /diff_param/ 
     +  sh0_diff, sh0_lps, lr_sh_param 

      real min_mass 	! mass threshold (%) 
      common /mass_param/ 
     +  min_mass 

c	output matrices
 
      integer dtinteg			! current total integration/averaging time interval (hr)
      real c_fields(maxx_c,maxy_c,nspec,12)   	! conc matrices 
      real conc_avg(maxx_c,maxy_c,12)
      integer navgstep(12)

      common /c_dep_matrices/ 
     +  dtinteg,
     +  c_fields,conc_avg,
     +  navgstep
 
c	misc params

      real ex2(41)		! values of exp(x**2/2), for x = 0, 0.1, 0.2, 0.3, .. 4
      common /misc_params/
     +  ex2

c	missing data flags

      integer imissing
      real xmissing
      parameter (imissing = -999, xmissing = -999.)
      
c	QA/QC

      character*21 progname
      parameter (progname = 'ATMOS 4.0 for SIM-air')

c       Deposition and Reaction Rates

      real dry_vd_p10, wet_vd_p10
      real dry_vd_p25, wet_vd_p25
      real dry_vd_so2, wet_vd_so2
      real dry_vd_so4, wet_vd_so4
      real dry_vd_nox, wet_vd_nox
      real dry_vd_no3, wet_vd_no3
      real dry_vd_1, wet_vd_1
      real dry_vd_2, wet_vd_2

      real kr_so2, kr_nox
      real kr

      common /rates/ 
     +   dry_vd_p10, wet_vd_p10, dry_vd_p25, wet_vd_p25,
     +   dry_vd_so2, wet_vd_so2, dry_vd_so4, wet_vd_so4,
     +   dry_vd_nox, wet_vd_nox, dry_vd_no3, wet_vd_no3,
     +   dry_vd_1, wet_vd_1, dry_vd_2, wet_vd_2,
     +   kr_so2, kr_nox, kr
